package memory.modele;

import java.lang.reflect.Array;

import memory.controleur.Controleur;
import memory.controleur.ControleurFenetre;
import memory.vue.FenetreJeu;
import memory.vue.FenetrePrincipale;
import memory.vue.ThreadOrdinateur;

import javax.swing.JFrame;
import javax.swing.JLabel;
/**
 * Classe Modele contenant les donnes du plateau
 * @author Anthony Laplane
 */
public class PlateauModele {
	// champs d'instance
	private int nbLignes;
	private int nbColonnes;
	private int nbTentatives;
	private int nbPaires;

	private FenetrePrincipale parentPartie;
	//private NiveauTresFacile niveau;
	private INiveau niveau;
	private int[]tabCartes;
	// Variables memorisant les cartes retourns
	// la valeur par defaut est -1 car 0 est un valeur existance
	private int positionCarte1 = -1;
	private int positionCarte2 = -1;
	
	private int varDifficulte;
	private int varNiveau;
	private int modeJeu;
	private int joueurEnCours;
	// Tableau mmorisant l'ensemble des cartes et leurs positions
	public int[][] positionCartes;
	
	
	/**
	 * Mthode lancer lors de l'instanciation de la classe PlateauModele
	 * @param parent   		Fenetre principale de l'application
	 * @param varDifficulte Niveau de difficult de la partie
	 * @param varNiveau     Numro du niveau, ou stage
	 */
	public PlateauModele(FenetrePrincipale parent, int varDifficulte, int varNiveau, int modeJeu){
		parentPartie = parent;
		this.varDifficulte = varDifficulte;
		this.varNiveau = varNiveau;
		this.modeJeu  = modeJeu;
		this.joueurEnCours = 1;
		// On verifie que le niveau existe lors de sa creation
		int niveauExiste = definirNiveau(varDifficulte,varNiveau);
		if (niveauExiste==1) {
			definirTentatives (retournerNbCases());
			creerTableauCartes(retournerNbCases());
			melangerTableau(tabCartes, retournerNbCases());
			int i;
			for(i=0; i<tabCartes.length;i++) {
				System.out.println("PLATEAU CARTE " +(i+1) + " :  "+tabCartes[i]);
				
			}
			
			definirPositionCartes(retournerNbCases());
			construirePlateauVue();
			creerJoueurs(modeJeu);
			// On stocke le tableau de cartes
			positionCartes = Controleur.monControleurPartie.positionCartes;
		}
	}
	
	
	// mthodes d'instance
	/**
	 * Associe le niveau de difficult contenu dans varDifficulte  un texte
	 * @return texteDiffulte	retourne le texte en type String
	 */
	public String retournerDifficulteText () {
		String texteDiffulte = new String();
		switch(varDifficulte){
			case 1 :
				texteDiffulte = "Trs facile";
			break;
			case 2 :
				texteDiffulte = "Facile";
			break;
			case 3 :
				texteDiffulte = "Normal";
			break;
			case 4 :
				texteDiffulte = "Difficile";
			break;
		}
		return texteDiffulte;
	}
	/**
	 * retourne le joueur en cours : 1 = Joueur1, 2 = Joueur2, 3 = Ordinateur
	 * @return modeJeu	Numro de mode du jeu de type entier
	 */
	public int retournerJoueurEnCours () {
		return this.joueurEnCours;
	}
	/**
	 * retourne le mode du jeu : 0 = 1 joueurs, 1 = 2 joueurs, 2 = contre l'ordinateur
	 * @return modeJeu	Numro de mode du jeu de type entier
	 */
	public int retournerModeJeu () {
		return this.modeJeu;
	}
	/**
	 * retourne le niveau de difficult du jeu
	 * @return varDifficulte	Numro de difficulte de type entier
	 */
	public int retournerDifficulte () {
		return this.varDifficulte;
	}
	/**
	 * retourne le numro du niveau de jeu
	 * @return varNiveau	Numro de niveau ou stage de type entier
	 */
	public int retournerNiveau () {
		return this.varNiveau;
	}
	/**
	 * Retourne le nombre de lignes du plateau
	 * @return nbLignes 	Nombre de lignes de type entier
	 */
	public int retournerNbLignes () {
		return nbLignes;
	}
	/**
	 * Retourne le nombre de colonnes du plateau
	 * @return nbColonnes 	Nombre de colonnes de type entier
	 */
	public int retournerNbColonnes () {
		return nbColonnes;
	}
	/**
	 * Retourne le nombre de tentatives
	 * @return nbColonnes 	Nombre de tentatives de type entier
	 */
	public int retournerTentatives () {
		return nbTentatives;
	}	
	/**
	 * Retourne le nombre de cases du plateau
	 * @return nbColonnes 	Nombre de cases de type entier
	 */
	public int retournerNbCases () {
		//nbCartes =nbLignes*nbColonnes;
		return nbLignes*nbColonnes;
	}
	/**
	 * Retourne la valeur de la carte  partir de sa position
	 * @return tabCartes 	Valeur de la carte de type entier
	 */
	public int retournerNumCarte(int position) {
		return tabCartes[position];
	}
	/**
	 * Calcul et retourne le score
	 * @return score 	Score obtenu lors d'une partie russi
	 */
	public int retournerScore() {
		return nbTentatives*10;
	}
	/**
	 * Mmorise la carte retourne en dans positionCarte1 ou positionCarte2
	 * selon qu'une carte est dj retourne ou pas
	 * @param positionCarte   position de la carte
	 */	
	public void retournerCarte(int positionCarte) {

		if (this.positionCarte1 == -1) {
			this.positionCarte1 = positionCarte;
			System.out.println("plateau retourner " + positionCarte1);			
		} else {
			this.positionCarte2 = positionCarte;
			System.out.println("plateau retourner 2 " + positionCarte2);
			testerCartes();
		}

	}
	/**
	 * Test le couple de carte afin de verifier si il forme une paire
	 * Cette mthode reseigne testCartes en lui donnant la valeur :
	 * 1 si les cartes ne forment pas une paire
	 * 2 si les cartes forment pas une paire,
	 * 3 si c'est une paire et qu'il n'y a plus de carte, 
	 * 4 si le joueur a depasser le nombre de tentative
	 * nombre de tentative
	 */
	void testerCartes(){
		int valeurCarte1 = retournerNumCarte(this.positionCarte1);
		int valeurCarte2 = retournerNumCarte(this.positionCarte2);
		System.out.println("valeur 1 : " +valeurCarte1);
		System.out.println("valeur 2 : " +valeurCarte2);
		diminuerTentatives();
		System.out.println("Tentatives : " +nbTentatives);

			if (valeurCarte1 == valeurCarte2) {
				this.nbPaires = this.nbPaires + 1;
				// On test si toutes les paires ont t retournes
				if (this.nbPaires == retournerNbCases()/2) {
					System.out.println("FIN JEU                     !!!");
					// On incremente le score du joueur
					incrementerNbPairesReussies(retournerJoueurEnCours());
					Controleur.monControleurPartie.testCartes = 3;
				} else {
					// Le joueur a fait une paire
					System.out.println("paire!!!");
					//Controleur.monControleurPartie.testPartie();
					// On incremente le score du joueur
					incrementerNbPairesReussies(retournerJoueurEnCours());
					Controleur.monControleurPartie.testCartes = 2;
					// Si on joue contre l'ordinateur, on marque ces cartes comme ne faisant plus partie du jeu
					if (retournerModeJeu()==2) {
						MemoryModele.Ordinateur1.enleverCarteOrdinateur(positionCarte1, positionCarte2);
						//MemoryModele.Ordinateur1.enleverCarteOrdinateur(retournerNumCarte(positionCarte2), positionCarte2);
					}
					
				}
			} else {
				if (nbTentatives==0) {
//					 Le joueur n'a pas fait de paire et il n'y a plus de carte
					Controleur.monControleurPartie.testCartes = 4;
				} else {
					// Le joueur n'a pas fait de paire
					Controleur.monControleurPartie.testCartes = 1;
					// Si on joue contre l'ordinateur, il memorise le numero carte et sa position
					if (retournerModeJeu()==2) {
						MemoryModele.Ordinateur1.memoriserCarteOrdinateur(retournerNumCarte(positionCarte1), positionCarte1);
						MemoryModele.Ordinateur1.memoriserCarteOrdinateur(retournerNumCarte(positionCarte2), positionCarte2);
					}
					
				}
			}
		
		this.positionCarte1 = -1;
		this.positionCarte2 = -1;
	}
	/**
	 * Definit le tableau qui va contenir la position des cartes 
	 * @param nbCases   nombre de cases du plateau
	 */
	void definirPositionCartes (int nbCases) {
		positionCartes = new int[nbCases][2];
		//Controleur.monControleurPartie.positionCartes = new int[nbCases][2];
	}

	/**
	 * Stock les coordonnes de la carte 
	 * @param nbCases   numero de la carte
	 * @param ligne   ligne de la carte
	 * @param colonne   colonne de la carte
	 */
	public void stockerPositionCarte (int numCarte, int ligne, int colonne) {
		//positionCartes[numCarte][0] = ligne;
		//positionCartes[numCarte][1] = colonne;
		//System.out.println("CARTE " +numCarte+ " ligne :" + positionCartes[numCarte][0]);
		//System.out.println("CARTE " +numCarte+ " colonne :" + positionCartes[numCarte][1]);
	}
	/**
	 * Definit le nombre de lignes du plateau en le stockant dans la proprit nbLignes
	 * @param varNbLignes   nombre de ligne du plateau de type entier
	 */
	void definirNbLignes (int varNbLignes) {
		nbLignes = varNbLignes;
	}
	/**
	 * Definit le nombre de colonnes du plateau en le stockant dans la proprit nbColonnes
	 * @param varNbColonnes   nombre de colonnes du plateau de type entier
	 */
	void definirNbColonnes (int varNbColonnes) {
		nbColonnes = varNbColonnes;
	}
	/**
	 * Definit le nombre de tentatives en fonction de la taille du niveau et de la difficult
	 * @param nbCases   nombre de nbCases du plateau de type entier
	 */
	void definirTentatives (int nbCases) {
		nbTentatives = nbCases*(5-varDifficulte)/4+nbCases/2;
	}	

	/**
	 * Diminue le nombre de tentative, la proprit nbTentatives
	 */
	void diminuerTentatives () {
		nbTentatives = nbTentatives-1;
	}
	/**
	 * Cre le tableau tabCartes qui va contenir les paires de cartes
	 * @param nbCases   nombre de nbCases du plateau de type entier
	 */
	void creerTableauCartes (int nbCases) {
		//System.out.println(" Nombre Cases " + nbCases);
		tabCartes = new int[nbCases];
		int i;
		int carte = 1;
		for (i=0;i<nbCases-1;i=i+2) {
			//System.out.println(i);
			tabCartes[i] = carte;
			//System.out.println("Carte " + tabCartes[i]);
			tabCartes[i+1] = carte;
			//System.out.println("Carte " + tabCartes[i]);
			carte = carte + 1;	
			
		}
	}
	/**
	 * Inverse deux elements dans un tableau
	 * @param tableau[]   	Le tableau su lequel l'inversion va tre faite
	 * @param numElement1   numro de ligne de l'lement 1 de type int
	 * @param numElement2   numro de ligne de l'lement 1 de type int
	 */
	void inverserElements (int tableau[], int numElement1, int numElement2) {
		int valeurTemp = tableau[numElement1];
		tableau[numElement1] = tableau[numElement2];
		tableau[numElement2] = valeurTemp;
		
	}
	/**
	 * Retourne un nombre alatoire compris entre nombreMini et nombreMaxi
	 * @param nombreMini   nombre de type entier qui dfinit la valeur minimum qui doit etre retourne
	 * @param nombreMaxi   nombre de type entier qui dfinit la valeur maximum qui doit etre retourne
	 * @return int		 	Retourne le nombre aleatoire
	 */
	int nombreAleatoire(int nombreMini, int nombreMaxi){
		return (int)(Math.random() * (nombreMaxi-nombreMini)) + nombreMini; 
	}
	/**
	 * Melange le tableau x fois ou x est le nombre de Case, soit le nombre totale de cartes
	 * @param tabCartes   tableau qui contient les cartes
	 * @param nbCases     nombre de Cases du plateau
	 * @return int		 	Retourne le nombre aleatoire
	 */
	void melangerTableau(int tabCartes[], int nbCases){
		int i;
		for (i=0;i<nbCases;i++){
			inverserElements(tabCartes,nombreAleatoire(0,nbCases),nombreAleatoire(0,nbCases));
			//System.out.println("nb inv : " + i);
		}
	}
	/**
	 * Instancie FenetreJeu qui est la vue
	 */
	void construirePlateauVue () {
		ControleurFenetre.maFenetreJeu = new FenetreJeu(parentPartie, this);
	}
	/**
	 * Definit le nombre de joueur en fonction du mode de jeu
	 * @param modeJeu	 Mode de jeu de type entier
	 */
	void creerJoueurs(int modeJeu) {
		
		// On cre dans tous les cas le premier joueur
		// et en fonction du mode de jue le joueur2 ou l'ordinateur
		MemoryModele.Joueur1 = new Joueur();
		switch (modeJeu)
		{
		case 1 :
			MemoryModele.Joueur2 = new Joueur();			
		break;
		case 2 :
			MemoryModele.Ordinateur1 = new Ordinateur();
			// On definit le niveau de difficult de l'ordinateur
			MemoryModele.Ordinateur1.definirDifficulteOrdinateur(this.varDifficulte);
			// On defintit la taille de la mmoire
			MemoryModele.Ordinateur1.definirMemoireOrdinateur(retournerNbCases());
			// On instancie le thread de l'ordinateur
			ControleurFenetre.threadOrdinateur1 = new ThreadOrdinateur();
			ControleurFenetre.threadOrdinateur1.start();
			
		break;
		
		}
	}
	/**
	 * Ajouter une paire de carte au compteur du joueur
	 * @param joueurEnCours  le numero du joueur en cours de type entier
	 */
	void incrementerNbPairesReussies(int joueurEnCours){
		switch (joueurEnCours)
		{
		case 1 :
			MemoryModele.Joueur1.incrementerNbPairesReussies();			
		break;
		case 2 :
			MemoryModele.Joueur2.incrementerNbPairesReussies();	
		break;
		case 3 :
			MemoryModele.Ordinateur1.incrementerNnPairesReussies();
		break;
		
		}
		
	}
	/**
	 * Definit le joueur qui va pouvoir jouer
	 * @param modeJeu  Mode de jeu en cours de type entier
	 */
	public void changerDeMain(int modeJeu) {
		switch (modeJeu)
		{
		case 1 :
			if (joueurEnCours == 1) {
				joueurEnCours = 2;
			} else {
				joueurEnCours = 1;
			}	
		break;
		case 2 :
			if (joueurEnCours == 1) {
				joueurEnCours = 3;
				ControleurFenetre.threadOrdinateur1.joueurEnCours = 3;
				
			} else {
				joueurEnCours = 1;
				ControleurFenetre.threadOrdinateur1.joueurEnCours = 1;
			}	
		break;
		
		}
		
	}
	
	/**
	 * Definit le nombre de ligne et de colonne en fonction du niveau de difficult
	 * et du niveau ou stage
	 * @param varDifficulte Niveau de difficult de la partie
	 * @param varNiveau     Numro du niveau, ou stage
	 */
	int definirNiveau(int varDifficulte, int varNiveau){
		// instancie la classe du niveau correspondant  la difficult
		switch (varDifficulte)
		{
		case 1: 
			niveau = new NiveauTresFacile();
		break;
		case 2: 
			niveau = new NiveauFacile();
		break;
		case 3: 
			niveau = new NiveauNormal();
		break;
		case 4: 
			niveau = new NiveauDifficile();
		break;
		}
		// appel des methodes permettant de stocker le nombre de ligne et de colonne
		definirNbLignes(niveau.retournerNbLignesNiveau(varNiveau));
		definirNbColonnes(niveau.retournerNbColonnesNiveau(varNiveau));
		if (nbLignes==0) {
			Controleur.monControleurNouvellePartie.terminerJeu();
			return 0;
		} else {
			return 1;
		}
		
	}
}
